//***************************************************************************
//  init.c
//***************************************************************************

//***************************************************************************
//								PIC18F2550
//						    +------------------+
//			  	  RESET --> | 1 MCLR	 RB7 28| <-- IR_LEFT
//			 	  LineL --> | 2 RA0/AN0	 RB6 27| <-- BTN
//				  LineR --> | 3 RA1/AN1	 RB5 26| <-- IR_RIGHT
//				        --> | 4 RA2/AN2	 RB4 25| --> IR_CTRL 
//				        --> | 5 RA3/AN3	 RB3 24| --> MOTB_DIR1
//	 		            --> | 6 RA4		 RB2 23| --> MOTB_DIR2
//	   	 		        --> | 7 RA5		 RB1 22| --> MOTA_DIR2
//					GND	--> | 8 VSS		 RB0 21| --> MOTA_DIR1
//	 			   OSC1 <-- | 9 		 VDD 20| <-- 5 volts
//	 			   OSC2 <-- |10 		 VSS 19| <-- GND
//		  HEARTBEAT_LED <-- |11 RC0		 RC7 18| <-- UART:RX
//		       MOTA_PWM <-- |12 CCP2	 RC6 17| --> UART:TX
//		       MOTB_PWM <-- |13 CCP1	 RC5 16| --> D+
//				   Vusb <-- |14 Vusb	 RC4 15| --> D-
//						    +------------------+
//
//***************************************************************************
//  0 = port pin is an output
//  1 = port pin is an input
//  x = don't care
//  "I/O Ports" chapter in datasheet.
//***************************************************************************
void initPorts(void){
  //        76543210
  TRISA = 0b00000011;  // LineR, LineL 
  TRISB = 0b11100000;  // IR_LEFT, BTN, IR_RIGHT, IR_CTRL, MOTB_DIR1, MOTB_DIR2, MOTA_DIR2, MOTA_DIR1
  TRISC = 0b10000000;  // UART:RX, UART:TX, D+, D-, Vusb, MOTB_PWM, MOTA_PWM, HEARTBEAT_LED 
  
  LED1=0; LED2=0; // turn off LEDs by default
}// End of initPorts()

//***************************************************************************
//  initUART
//***************************************************************************
void initUART(unsigned int baudRate){
  TXSTA = 0b00100100;    // 8-bit TX, asynchronous mode, high speed baudrate
  RCSTA = 0b10010000;    // Enable serial port, 8-bit RX
  BAUDCON = 0b00000000;  // 8-bit baudrate generator
  SPBRG = baudRate;
}

//***************************************************************************
// Timer0 generates an interrupt every 1 ms which is used to:
//   1) increment the systemTime variable,
//   2) produce delays of multiple milliseconds, delay_ms(),
//   3) ramp the motor speeds up and down.
// Fcy = 12 MHz => Tcy = 83.33 ns
// Prescaler = 1:2 => Timer0 is clocked at 166.7 ns
// 166.7 ns x 6000 = 1 ms
// Load timer0 with 65536 - 6000 = 59536 = 0xE890 so it overflows and 
//   interrupts every 1 ms.
//***************************************************************************
void initRTI(void){
  T0CON = 0b10000000;
  //        ^^^^^^^^
  //        |||||+++-- Pre-scaler = 1:2
  //        ||||+----- Pre-scaler assigned
  //        |||+------ don't care
  //        ||+------- Clock source = Fcy
  //        |+-------- 16-bit timer
  //        +--------- Enable timer
  INTCON2bits.TMR0IP = 0;    // Low priority
  TMR0H = 0xE8;
  TMR0L = 0x90;
  systemTime = 0;
}

//***************************************************************************
// Fcy = 12 MHz => Tcy = 83.33 ns
// Prescaler = 1:4 => Timer0 is clocked at 333.333333333333ns
//  3 of them makes 1us... used for timing things. 
// We will use this for timing very short delays
//***************************************************************************
void initT1(void){
  T1CON = 0b10100001;
  PIE1bits.TMR1IE=0; // disable interrupts on this line
}

void initMotors(void)
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
//	initMotors
//		CCP1 generates MOTB_PWM  and  CCP2 generates MOTA_PWM.
//	    PWM period = (PR2 + 1) x 4 x (1/Fosc) x Prescaler 
//      PWM period = (47 + 1) x 4 x (1/48) x 16 = 64 us => 15,625 Hz PWM freq
//		Resolution = log(4 x (PR4 + 1)) / log(2) = 10 bits
//	
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
{
	MOTA_DIR1 = 0;	// Put motor drivers into high impedance state, ie. OFF
	MOTA_DIR2 = 0;
	MOTB_DIR1 = 0;
	MOTB_DIR2 = 0;
	
    PR2 = 00101111;       // Set the PWM period to 15.625kHz (PR2=47)
	CCP1CON = 0b00001100; // CCP1 PWM mode enabled
    CCP2CON = 0b00001100; // CCP2 PWM mode enabled
    CCPR1L = 0;           // Duty cycle = 0%  
    CCPR2L = 0;           // Duty cycle = 0%
    PIR1bits.TMR2IF = 0;  // Clear timer2 interrupt flag
    T2CON = 0b00000111;   // Configure timer2: Prescaler = 1:16, timer enabled
		
    while(!PIR1bits.TMR2IF);// Wait for timer2 to match PR2 and set the interrupt flag
	} // End of initMotors()

//***************************************************************************
//    initADC
//
//    ADC     Line
//    chnl    Detector
//    ~~~~~~~~~~~~~~~~~
//    AN0   =  LineL
//    AN1   =  LineR
//***************************************************************************
void initADC(void){

  ADCON0 = 0b00000000;  // Select AN0, disable ADC module
  ADCON1 = 0b11111100;  //  Only setting RA0 and RA1 to analog inputs.
  ADCON2 = 0b00111110;  // Result is left justified, Acquistion time = 20Tad, ADC clk = Fosc/64.
  IPR1bits.ADIP = 1;    // High priority interrupt
  PIR1bits.ADIF = 0;    // Clear interrupt flag
  PIE1bits.ADIE = 0;    // Disable interrupts
  ADCON0bits.ADON = 1;  // Enable the ADC module
}// End of initADC()
