//***************************************************************************
//  main.c
//***************************************************************************
#include <p18f2550.h>
#include "main.h"
#include <stdio.h>
#include <timers.h>

//***************************************************************************
// Local Prototypes 
//***************************************************************************
void delay_ms(unsigned int duration);
void delay_us(unsigned char duration); // sub 255us delays only!
void delay_us_long(unsigned int duration); // horribly inaccurate for sub 10us
void LowPriorityTable(void);
void HighPriorityTable(void);
signed char button(void);
void initMotors(void);
void disableMotors(void);
void setMotorDirection(unsigned char whichMotor, unsigned char direction);
void motorControl(unsigned char whichMotor, unsigned char direction, unsigned char speed, signed char acceleration);
unsigned char 	leftMotorTargetSpeed, rightMotorTargetSpeed;	// Target speeds 
unsigned char 	leftMotorAccel, rightMotorAccel;				// Allowed accelerations
unsigned char leftRange, rightRange, centreRange;
unsigned char leftContact, rightContact;
unsigned char checkEdges;
unsigned char  ObjectStraightAhead;

//***************************************************************************
//***************************************************************************
extern void _startup (void);        // See c018i.c in your C18 compiler dir
#pragma code REMAPPED_RESET_VECTOR = 0x1000
void _reset (void){
  _asm goto _startup _endasm
}

//***************************************************************************
// handy global variables
//***************************************************************************
unsigned long systemTime;  // system tick, increments every 1ms
unsigned char random; // a random number based on A/D readings

struct {
  unsigned  enableAlarm      :1; // Alarm condition, blink ALARM_LED rapidly
  unsigned  unused_1         :1;  
  unsigned  motorRampEnabled :1; // Timer0 interrupt is allowed to adjust the motors
  unsigned  enableHeartbeat  :1; // Enable HEARTBEAT_LED
  unsigned  unused_4         :1; // enable interrupt control of LCD buffers
  unsigned  unused_5    	 :1;
  unsigned  unused_6         :1;
  unsigned  unused_7         :1;
} flags;

//***************************************************************************
// include our other various bits of source code
//***************************************************************************
#include "interrupts.c" // interrupt handlers
#include "init.c"       // initialization code 
#include "motors.c"   // Motor Control 
#include "sensors.c"   // Object detection
#include "smashndash.c"   // main definition for Smash 'n Dash

//***************************************************************************
// Long Delays
//    Resolution = RTI rate 1 ms
//    Realtime interrupt MUST be running
//***************************************************************************
void delay_ms(unsigned int duration){
  unsigned long endTime;  
  endTime = systemTime + duration - 1;
  while(systemTime <= endTime);
} // End of longDelay()

//***************************************************************************
// Short Delay
//   use Timer1 to wait this amount of time (in microseconds). Not overly
//   accurate.... sub 255us delays. +-2us perhaps?
//***************************************************************************
void delay_us(unsigned char duration){
  unsigned char endTime;
  // run the loop 3 times... lets us get up to 255us delay. Its more 
  // efficient (and more accurate) to do a for loop, but then we'd
  // be limited to around 80us
  TMR1H=0; TMR1L=0; // reset timer to 0
  while(TMR1L<duration);
  TMR1L=0; // reset timer to 0
  while(TMR1L<duration);
  TMR1L=0; // reset timer to 0
  while(TMR1L<duration);
}

//***************************************************************************
// Short Delay - longer version
// use Timer1 to wait this amount of time (in microseconds). Not overly
//   accurate, particularly at sub 20us. +-5us perhaps?
//***************************************************************************
void delay_us_long(unsigned int duration){
  unsigned int endTime;
  TMR1H=0; TMR1L=0; // reset timer to 0
  if (duration>9) duration=duration-9;
  endTime=duration*3; // takes 3 ticks to make 1us
  while(ReadTimer1()<endTime); // speed this up by ditching function call....
}
//***************************************************************************

void main(void){
  unsigned char i=0;
  unsigned long endTime;
  static unsigned int heartbeat=HEARTBEAT_RATE, alarm=ALARM_RATE;
  // Configure the oscillator. "Oscillator Module (With Fail-Safe Clock Monitor)" chapter in datasheet.
  OSCCON  = 0b00000000;      // Bits 1 & 0 = 00 => select primary oscillator.
  while(!OSCCONbits.OSTS);    // Wait for oscillator timer to expire.
  
  // Initialize the various subsystems.
  initPorts();          // Initialize the port pin directions.
  initUART(BAUD_19200);      // Initialize the UART
  initRTI();            // Initialize Timer0 (realtime interrupt).
  initT1(); // initialize Timer1
  initADC();            // Initialize the ADC module.
  initMotors();	 // Initialize CCP1 and CCP2 in PWM mode

  //  Initialize the variables
  disableMotors();       // // Disable both motors.
  flags.enableHeartbeat=TRUE;  // Ebable heartbeat
  flags.enableAlarm=FALSE;  // No alarm condition
  fsmInitialize ();
  chnlIndex = 0;       // Initialize lineChnls[] index.
  ADCON0bits.CHS0 = 0; // First channel to be converted is 0.
  ADCON0bits.CHS1 = 0;
  ADCON0bits.CHS2 = 0;
  ADCON0bits.CHS3 = 0;
  ADCON0bits.GO = 1;   // Start an ADC conversion.
  
  IR_ENABLE = 0; 
  
  // Enable interrupts.
  // DO NOT USE DELAY FUNCTIONS BEFORE THIS POINT!
  INTCONbits.TMR0IE = 1; // Enable Timer0 (RTI) interrupt.
  PIE1bits.ADIE = 1;     // Enable ADC interrupts.
  RCONbits.IPEN = 1;     // Enable interrupt priorities.
  INTCONbits.GIEH = 1;   // Enable high priority interrupts.
  INTCONbits.GIEL = 1;   // Enable low priority interrupts.

  for(;;){	
	delay_ms(10);
	ReadSensors ();
	fsmExecute ();
}
 
  // should never get here, but just in case....
  // End of the road, blinky blinky forever
  for(;;){
    LED1=1; LED2=0;
    delay_ms(500);
    LED1=0; LED2=1;
    delay_ms(500);
  }  
} // End of main()


